# 
#       #=======================================================#     _\|/_
#      #  __  _____           ___                    ___       #       /|\ ~
#     #  /      |      ^     |   \  |         ^     |   \     #           _\|/_
#    #   \__    |     / \    |___/  |        / \    |___/    #             /|\ ~
#   #       \   |    /___\   |  \   |       /___\   |   \   #  _\|/_
#  #     ___/   |   /     \  |   \  |____  /     \  |___/  #    /|\ ~
# #                                                       #             _\|/_
##=======================================================#               /|\ ~
#
#
##  Makefile.worker.inc: contains boilerplate rules for
##                       lowest-level Starlab makefiles.
##.............................................................................
##    version 1.1:	Dec 1993	Piet Hut & Steve McMillan
##.............................................................................
#
#   All macros not explicitly defined below are assumed to be declared
#   in the local makefile being used.
#
#   Local (i.e. non-environment) macros used in Makefile.worker.inc:
#
#	OBJECT_FILES		files to be placed in library LIBRARY
#				by 'make lib'
#	LIBRARY			library to hold OBJECT_FILES
#	EXECUTABLES		C++ executables to be built by 'make all'
#	NON_CCC_EXECUTABLES	non-C++ executables to be built by 'make all'
#	DEPEND_LIBS		libraries on which executables depend
#
#	CFLAGS			flags to be passed to C/C++ compilers
#	CPPFLAGS		C preprocessor flags (passed to the C
#				compiler only)
#
#	LIBFLAGS		library-related switches (passed to loader)
#	EXEFLAGS		other flags to be passed to loader
#
#..............................................................................

SHELL = /bin/sh

RM = /bin/rm
LS = /bin/ls
CP = /bin/cp

BIN = $(STARLAB_PATH)/bin
INC = $(STARLAB_PATH)/inc
LIB = $(STARLAB_PATH)/lib

INC_STAR = $(STARLAB_PATH)/inc/star
TOOLFLAGS = -DTOOLBOX

SMAKE = ${MAKE} $(STARLAB_MAKEFLAGS)

.KEEP_STATE:
.DELETE_ON_ERROR:

#..............................................................................

# if make is invoked without argument, the following help message is displayed:

help:
	@echo This Makefile offers the following make options:
	@echo '  sure: executes  make clean  and  make all'
	@echo '  all: executes  make lib  and  make bin'
	@echo '  lib: makes the library, if any, in this directory'
	@echo '  bin: makes all executables in this directory'
	@echo \
       '  clean: removes executables, object files, backup files and core file'
	@echo STARLAB_PATH = $(STARLAB_PATH)

#..............................................................................

sure:	clean all

#..............................................................................

all:
	@$(SMAKE) lib
	@$(SMAKE) bin

# Separate lib and bin here to force lib to be made in its entirety
# before bin is started.

#..............................................................................

# Use make_touch_src here to make sure that zero-length versions
# of nonexistent source files are created prior to the compilation
# (to avoid errors from make...).

# The null files are neither compiled nor placed in the library.
# They are deleted after use.

LOCAL_OBJECT_FILES = $(OBJECT_FILES)
#			             # dummy.o	# To fool make into silence.
#				     # ^^^^^^^    (May no longer be needed...)

# If we retain dummy.o, then we will have to modify the logic if the
# dependency of .bin on .lib is to be retained (dummy.C doesn't exist
# when we get to "make .lib" via "make .bin"...).

lib:
	$(STARLAB_PATH)/sbin/make_touch_src .C $(LOCAL_OBJECT_FILES)
	@$(SMAKE) .lib

.lib:	$(LOCAL_OBJECT_FILES)
	$(STARLAB_PATH)/sbin/make_ar_cleanup $(LIB) "$(LIBRARY)" $(LOCAL_OBJECT_FILES)

#..............................................................................

# The same comments about missing source files apply here too...

# The dependency on .lib forces libraries to be built completely
# before binaries are made.

bin:
	$(STARLAB_PATH)/sbin/make_touch_src .C $(OBJECT_FILES)
	@$(SMAKE) .bin

.bin:	.lib $(EXECUTABLES) $(NON_CCC_EXECUTABLES)
	@touch .bin

list_obj:
	@echo $(OBJECT_FILES)

list_exe:
	@echo $(EXECUTABLES)

list_exe_src:
	@for file in $(EXECUTABLES) ; do echo -n $$file.C " "; done ; echo " "

#..............................................................................

# >>>>> Discard all implicit rules, and reestablish some of our own. <<<<<

.SUFFIXES:
.SUFFIXES:	.C .c .f .F .o

# Override the default .C, .o, and .C.o rules (note the additional
# dependencies in .C).

# The files listed in DEPEND_LIBS (if any are specified) MUST exist.
# Otherwise, the rule will NOT be recognized and the make will fail.


# WARNING!!!  The following target allows executables to depend both
# on their source files and on several libraries, so programs are
# rebuilt if the library changes.  However, it may cause some versions
# of make to go into an infinite loop!  GNU make seems to recognize
# the recursion and avoids it properly ...most of the time...

#%:	%.C $(DEPEND_LIBS)

.C:
	$(STARLAB_PATH)/sbin/make_cond_build $@ $(BIN)/$@ \
		"$(LIBFLAGS) " $(STARLAB_CCC) $(CFLAGS) $(TOOLFLAGS) $(EXEFLAGS)


# Also override the default .o rule (to pick up .C, not .c!).
# In Starlab, we never build from the object file, even if one exists.

.o:
	$(STARLAB_PATH)/sbin/make_cond_build $@ $(BIN)/$@ \
		"$(LIBFLAGS) " $(STARLAB_CCC) $(CFLAGS) $(TOOLFLAGS)

.C.o:
	$(STARLAB_PATH)/sbin/make_cond_compile $< $(STARLAB_CCC) $(CFLAGS)

# Establish our own versions of the standard .c.o and .c rules.
# The default .f.o and .f rules are OK.

.c:
	$(STARLAB_C_COMPILE) $(CPPFLAGS) $(CFLAGS) $@.c -o $@ -lm

.c.o:
	$(STARLAB_C_COMPILE) $< -c $(CPPFLAGS) $(CFLAGS)

# include $(STARLAB_PATH)/local/Makefile.local.inc
# Not needed currently.

#..............................................................................

tarlist:
	@make_list_files `$(LS) [A-Z]* [A-Z,a-z]*.[CcFfh] | grep -v "~"`

#..............................................................................

clean:
	@-$(RM) -f core $(EXECUTABLES) $(NON_CCC_EXECUTABLES) *.o *~ .lib .bin \
		.make.state .make.dependency* $(EXTRA_CLEAN)

cleanbin:
	@-$(RM) -f $(EXECUTABLES) $(NON_CCC_EXECUTABLES)

cleanalt:

test:
	@env|grep STARLAB

#..............................................................................

##=======================================================================//
##  +---------------+        _\|/_        +------------------------------\\
##  |  the end of:  |         /|\         |  Makefile.worker.inc
##  +---------------+                     +------------------------------//
##========================= STARLAB =====================================\\ 
