#
#  starlab.sh
#
#  Source this file from Bourne-style shells (bash, sh, ksh):
#    . starlab.sh
#  or from e.g. ~/.bashrc, to ensure that the starlab environment is
#  visible during the "make" process.
#
#  However, the preferred route is to access this file only through the 
#  file "starlab_setup", as described in README.
#
#  You should work with a LOCAL COPY of this file, in order to avoid
#  having it overwritten each time the system is re-installed.
#
#  In addition, your path must be modified to see the Starlab environment.
#  Include the following lines in .cshrc or .login:
#
#     set path = ($path $STARLAB_PATH/sbin $STARLAB_PATH/bin)
#     rehash
#
#...........................................................................

setenv() {
  export $1="$2"
}
unsetenv() {
  unset $1
}

unsetenv	STARLAB_STAR
unsetenv	STARLAB_HYDRO
unsetenv	STARLAB_KEPLER
unsetenv	STARLAB_MCD
unsetenv	STARLAB_GFX
unsetenv	STARLAB_XUTIL
unsetenv	STARLAB_DYN
unsetenv	STARLAB_HDYN
unsetenv	STARLAB_SDYN
unsetenv	STARLAB_SDYN3
unsetenv	STARLAB_HARPSIMS
#
# Choose compilation options (***** IMPORTANT!! *****).
# Commenting everything out here will compile/tar only the minimal package.
#
# Keep everything on the "definitive" machine.
#
setenv	STARLAB_STAR	star 		# Stellar evolution
#setenv	STARLAB_HYDRO	hydro 		# Hydrodynamics
setenv	STARLAB_KEPLER	kepler 		# Kepler package (used by sdyn*)
#setenv	STARLAB_MCD	"libs draw" 	# Mcdraw graphics programs (need gfx)
setenv	STARLAB_GFX	gfx 		# Graphics library
setenv	STARLAB_XUTIL	xutil 		# X utilities (xstarplot uses gfx)
setenv	STARLAB_DYN	dyn 		# Dynamics programs
setenv	STARLAB_HDYN	hdyn 		# Hermite programs
#setenv	STARLAB_SDYN	sdyn 		# Multi-particle scattering
#setenv	STARLAB_SDYN3	sdyn3 		# Three-body scattering
#setenv	STARLAB_HARPSIMS harpsims	# HARP support

#...........................................................................

# Starlab-specific options and switches.
# -------------------------------------

# Compilers:

#	STARLAB_CCC			C++ compiler and necessary switches
#	STARLAB_C_COMPILE		C compiler
#	STARLAB_CPP			C preprocessor
#	STARLAB_FORTRAN_COMPILE		FORTRAN compiler
#	STARLAB_SYSTEM_C_COMPILE	"System" C compiler

# Compiler switches:

#	STARLAB_CCC_FLAGS		Optional C++ compiler flags
#	STARLAB_GFX_CFLAGS		Graphics-specific C-compiler flags

# These switches are general, but are presently only used in gfx:

#	STARLAB_FORTRAN_CFLAGS		Flags for C compiler with F77 objects
#	STARLAB_FORTRAN_FLAGS		FORTRAN compiler flags

#	STARLAB_CCC_LIBPATH		C++ library location (include "-L")
#	STARLAB_CCC_LIB			C++ library to use (include "-l")

# These flags are general, but the FORTRAN ones presently aren't used:

#	STARLAB_FORTRAN_LIBPATH		FORTRAN library location (include "-L")
#	STARLAB_FORTRAN_LIB		FORTRAN library to use (include "-l")

# We apparently don't need special treatment of the C libraries.

# Language combinations:

#	c++ ..... ccsub.o	OK -- no machine-dependent switches needed
#	c++ ..... f77sub.o	STARLAB_FORTRAN_LIBPATH, STARLAB_FORTRAN_LIB
#	cc  ..... c++sub.o	STARLAB_CCC_LIBPATH, STARLAB_CCC_LIB
#	cc  ..... f77sub.o	STARLAB_FORTRAN_LIBPATH, STARLAB_FORTRAN_LIB
#	f77 ..... c++sub.o	STARLAB_CCC_LIBPATH, STARLAB_CCC_LIB
#	f77 ..... ccsub.o	Nothing machine-dependent (but need -lm, etc.)

# Other useful information:

#	STARLAB_OS_FLAG			Define the operating system
#	STARLAB_RANLIB			Define "ranlib" for this machine
#	STARLAB_POSIX_LIB		Link with POSIX library, if any
#	STARLAB_X11INC			Location of X header files
#	STARLAB_X11LIB			Location of X libraries
#	STARLAB_XLIB			Additional X-related libraries
#	STARLAB_EXTRAS			Define any "extra" terms or options
#	STARLAB_MAKEFLAGS		Extra flags for make (e.g. parallel)
#	STARLAB_PVM_LIBS		How to use the PVM libraries (if any)

#...........................................................................

# Checklist:

# For Irix (guessed):

# setenv	STARLAB_CCC			"CC"
# setenv	STARLAB_C_COMPILE		"cc"
# setenv	STARLAB_CPP			"cc -E"
# setenv	STARLAB_FORTRAN_COMPILE		"f77"
# setenv	STARLAB_SYSTEM_C_COMPILE	"cc"
# setenv	STARLAB_CCC_FLAGS		"-n32"
# setenv	STARLAB_GFX_CFLAGS		""
# setenv	STARLAB_FORTRAN_CFLAGS		""
# setenv	STARLAB_FORTRAN_FLAGS		"-n32"
# setenv	STARLAB_CCC_LIBPATH		""
# setenv	STARLAB_CCC_LIB			""
# setenv	STARLAB_FORTRAN_LIBPATH		""
# setenv	STARLAB_FORTRAN_LIB		""
# setenv	STARLAB_OS_FLAG			"sgi"
# setenv	STARLAB_RANLIB			":"
# setenv	STARLAB_POSIX_LIB		""
# setenv	STARLAB_X11INC			"-I/usr/include/X11"
# setenv	STARLAB_X11LIB			"-lX11 -lXext"
# setenv	STARLAB_XLIB			""
# setenv 	STARLAB_EXTRAS			"-DHAS_USLEEP"
# setenv	STARLAB_MAKEFLAGS		""
# setenv	STARLAB_PVM_LIBS		""

#...........................................................................

# For Linux:

setenv	STARLAB_CCC			"g++ -DGNU_CPLUSPLUS"
setenv	STARLAB_C_COMPILE		"gcc"
setenv	STARLAB_CPP			"/usr/bin/cpp -C -Dlinux"
setenv	STARLAB_FORTRAN_COMPILE		"f77"
setenv	STARLAB_SYSTEM_C_COMPILE	"cc"
setenv	STARLAB_CCC_FLAGS		"-O -DHAS_PVM"
setenv	STARLAB_GFX_CFLAGS		" "
setenv	STARLAB_FORTRAN_CFLAGS		"-DFORTRAN_TRAILING_UNDERSCORE"
setenv	STARLAB_FORTRAN_FLAGS		" "
setenv	STARLAB_CCC_LIBPATH		"-L/usr/lib -L/usr/lib/gcc-lib/i386-redhat-linux/egcs-2.91.66"
setenv	STARLAB_CCC_LIB			"-lstdc++ -lgcc"
setenv	STARLAB_FORTRAN_LIBPATH		"-L/usr/lib/gcc-lib/i386-redhat-linux/egcs-2.91.66"
setenv	STARLAB_FORTRAN_LIB		"-lg2c"
setenv	STARLAB_OS_FLAG			"linux"
setenv	STARLAB_RANLIB			":"
setenv	STARLAB_POSIX_LIB		" "
setenv	STARLAB_X11INC			"-I/usr/X11R6/include/X11"
setenv	STARLAB_X11LIB			"-L/usr/X11R6/lib -lX11 -lXext"
setenv	STARLAB_XLIB			" "
setenv	STARLAB_EXTRAS			"-DHAS_USLEEP"
setenv	STARLAB_MAKEFLAGS		"-j 4"

if [ -n "$PVM_ROOT" -a -n "$PVM_ARCH" ]; then
    setenv  STARLAB_PVM_LIBS		"-L$PVM_ROOT/lib/$PVM_ARCH -lpvm3 -lgpvm3"
else

#   Actually OK to leave this unset, but better to set it explicitly.

    setenv  STARLAB_PVM_LIBS		" "

fi

#...........................................................................
