#
#  starlab_setup: This file should be invoked from .bashrc after
#		  setting the environment variable STARLAB_PATH.
#
#  This file MUST be sourced from .bashrc, not .bash_login, in order for
#  the starlab environment to be visible during the "make" process.
#
#-----------------------------------------------------------------------
#
if [ ! -n "$STARLAB_PATH" ]; then
    echo "STARLAB_PATH not set." >&2
    return 1
fi
#
# Check to see if a previous Starlab was in use.
#
if [ -n "$STARLAB_VERSION" ]; then
    prev=$STARLAB_VERSION
else
    prev=" "
fi
#
export STARLAB_VERSION=`basename $STARLAB_PATH`
#
starlab_shrc_file=$STARLAB_PATH/local/starlab.sh
#
if [ -f $starlab_shrc_file ]; then
    . $starlab_shrc_file
else
    echo No local/starlab.sh file found. >&2
fi
#
# Make sure a local Makefile exists.
#
if [ ! -d $STARLAB_PATH/local ]; then
    mkdir $STARLAB_PATH/local
fi
if [ ! -e $STARLAB_PATH/local/Makefile.local.inc ]; then
    touch $STARLAB_PATH/localMakefile.local.inc
fi
#
# Remove any old Starlab from the search path,
# but leave "/sbin", "/usr/sbin", "/usr/local/sbin" intact.
# Add Starlab to search path after . and .. (or at end of path if absent)
#
PATH=`echo "$PATH" | tr : '\12' | \
 awk '! /starlab.*bin$/ || /^(\/(|usr|local))+\/sbin$/ { p[++n] = $0; }
    /^\.+$/ { lastdot = n; }
    END {
	    if(lastdot == 0) lastdot = n;
	    p[lastdot] = p[lastdot] ":" spath "/bin:" spath "/sbin";
	    printf "%s", p[1];
	    for(i=2; i<=n; i++) printf ":%s", p[i];
    }' spath=$STARLAB_PATH`
export PATH
